﻿Shader "Refractive/LitRefractionSimplifiedClean2" {
	Properties {
		[Header(Color Params)]
		_EmissiveColor ("Emissive Color", Color) = (1,1,1,1)
		_Tint ("Color Intensity", range(0,3)) = 1		
		
		[Space]
		[Header(Surface params)]
		_Smoothness("Smoothness", range (0,1)) = 1
		[Toggle(_NORMAL_MAP)] _Normal_Map( "Enable Normal Mapping", Float ) = 0
		_NormalTex("Normal Texure", 2D) = "bump" {}
		
		[Space]
		[Header(Refraction Params)]
		_IOR("Index of Refraction (IOR)", float) = 1
		[Toggle(_USE_CHROMATIC_AB)] _Use_Chromatic_Ab("Use Chromatic Aberration", Float) = 0
		_CA("Chromatic Abberation (CA)", range(0,0.1)) = 0.1
		
		
	}
	SubShader {

		GrabPass{
		}
		// Removed <"RenderType"="Transparent"> Because this shader is using a GrabPass Anyway. 
		Tags {"Queue"="Transparent" }
		LOD 200

		CGPROGRAM
		// Physically based Standard lighting model, and enable shadows on all light types
		#pragma surface surf Standard fullforwardshadows vertex:vert

		// Use shader model 3.0 target, to get nicer looking lighting
		#pragma target 3.0
		#pragma shader_feature _NORMAL_MAP
		#pragma shader_feature _USE_CHROMATIC_AB

		sampler2D _GrabTexture;
		#if defined(_NORMAL_MAP)
			sampler2D _NormalTex;
		#endif
		fixed _IOR;
		fixed _CA;
		fixed _Tint;
		fixed _Smoothness;
		fixed4 _EmissiveColor;

		struct Input {
			float2 uv_MainTex;
			#if defined(_NORMAL_MAP)
				float2 uv_NormalTex;
			#endif
			float3 worldPos;
			float3 worldNormal; 
			INTERNAL_DATA
		};		

		void vert(inout appdata_full v, out Input o){
			UNITY_INITIALIZE_OUTPUT(Input,o);
		}

		fixed3 getRefracted(sampler2D tex, float3 worldViewDir, float3 worldNormal, float3 worldPos, fixed ior, fixed ca) {
				float3 worldRefractionDir = refract(worldViewDir, worldNormal, ior);
				float3 worldRefractionPos = worldPos.xyz + worldRefractionDir * ior;
				float4 clipRefractionPos = mul(UNITY_MATRIX_VP, float4(worldRefractionPos, 1.0));
				float4 coord = ComputeGrabScreenPos(clipRefractionPos);

				fixed3 result = tex2Dproj(tex, coord);

				#if defined (_USE_CHROMATIC_AB)
					result += fixed3(
						tex2Dproj(tex, coord + float4(-ca, 0.0, 0.0, 0.0)).r,
						tex2Dproj(tex, coord).g,
						tex2Dproj(tex, coord + float4(ca, 0.0, 0.0, 0.0)).b
					);
				#endif

			return result;
		}

		void surf (Input IN, inout SurfaceOutputStandard o) {
			fixed4 colorVal = _EmissiveColor;
			
			#if defined(_NORMAL_MAP)
				o.Normal = normalize(UnpackNormal( tex2D(_NormalTex, IN.uv_NormalTex) ) );
			#endif

			fixed ior = _IOR - 1;
			float3 worldViewDir = normalize( IN.worldPos.xyz );
			
			float roughness = 1.0 - _Smoothness;
			fixed3 bg = fixed3(0.0,0.0,0.0);
			bg = getRefracted( _GrabTexture, worldViewDir, WorldNormalVector (IN, o.Normal), IN.worldPos.xyz, ior, _CA)*_EmissiveColor;
			o.Albedo = fixed4(lerp(colorVal.rgb, bg, 1-colorVal.a)*_Tint, 1.0);
			o.Smoothness = _Smoothness;
			o.Metallic = 0;
		}
		ENDCG
	}
	FallBack "Diffuse"
}
